/**
   This program evaluates (interprets) an Abstract Syntax Tree (AST)
   that is defined by the following grammar.

       Prog ::= Exp
              | '(' 'prog' (Fun | Exp)+ Exp ')'

        Fun ::= '(' 'fun' VARIABLE Lambda ')'   // a function declaration

     Lambda ::= '(' 'lambda' VARIABLE* Exp ')'  // formal parameters followed by function body

        Exp ::= Array
              | IndexExp
              | Rand
              | For
              | IncDec
              | Apply
              | If
              | While
              | Begin
              | Set
              | Var
              | Print
              | AExp
              | BExp
              | INTEGER
              | BOOLEAN
              | VARIABLE

      Array ::= '(' 'array' 'dim' Exp ')'       // instantiate an array value of size Exp
              | '(' 'array'  Exp+ ')'           // instantiate an array value literal
              | '(' 'array' 'dim' Exp Exp* ')'  // partially initialized array

   IndexExp ::= '(' 'index' Exp Exp ')'     // 1st operand is array, 2nd operand is integer

     SizeOf ::= '(' sizeOf Exp ')'          // operand must be an array

       Rand ::= '(' 'rand' Exp Exp ')'      // return a random integer between Exp and Exp

        For ::= '(' 'for' Exp Exp Exp Exp ')'

     IncDec ::= '(' '++'  VARIABLE ')'      // post-increment
              | '(' '--'  VARIABLE ')'      // post-decrement
              | '(' '+++' VARIABLE ')'      // pre-increment
              | '(' '---' VARIABLE ')'      // pre-decrement

      Apply ::= '(' 'apply' Exp Exp* ')'    // function value followed by actual parameters

         If ::= '(' 'if' Exp Exp Exp ')'

      While ::= '(' 'while' Exp Exp ')'

        Set ::= '(' 'set' VARIABLE Exp ')'
              | '(' 'set' IndexExp Exp ')'  // set an array element

        Var ::= '(' 'var' VARIABLE Exp ')'

      Begin ::= '(' 'begin' Exp+ Exp ')'

      Print ::= '(' 'print' Exp ')'

       BExp ::= '(' '||'  Exp Exp+ ')'
              | '(' '&&'  Exp Exp+ ')'
              | '(' '!'   Exp ')'
              | '(' RelOp Exp Exp ')'
              | '('  EqOp Exp Exp ')'

      RelOp ::= '<' | '>' | '<=' | '>='
       EqOp ::= '==' | '!='

       AExp ::= '(' '+' Exp Exp* ')'
              | '(' '-' Exp Exp? ')'
              | '(' '*' Exp Exp+ ')'
              | '(' '/' Exp Exp  ')'
              | '(' '%' Exp Exp  ')'
              | '(' '^' Exp Exp  ')'

    INTEGER ::= [-][0-9]+
    BOOLEAN ::= 'true' | 'false'
   VARIABLE ::= [a-zA-Z][a-zA-Z0-9]*
*/

public class Language_7a extends Language_7
{

   // Evaluate an expression
   protected Value evaluateExp(Tree tree) throws EvalException
   {
      Value result = null;

      result = super.evaluateExp(tree);  // (CHANGE THIS)

      return result;
   }//evaluateExp()


   // Evaluate an array expression
   protected Value evaluateArray(Tree tree) throws EvalException
   {
      Value result = null;

      // Your implementation goes here.

      return result;
   }//evaluateArray()


   // Evaluate an index expression
   protected Value evaluateIndex(Tree tree) throws EvalException
   {
      Value result = null;

      // Your implementation goes here.

      return result;
   }//evaluateIndex()


   // Evaluate a sizeOf expression
   protected Value evaluateSizeOf(Tree tree) throws EvalException
   {
      Value result = null;

      // Your implementation goes here.

      return result;
   }//evaluateSizeOf()


   // Evaluate a rand expression
   protected Value evaluateRand(Tree tree) throws EvalException
   {
      Value result = null;

      // Your implementation goes here.

      return result;
   }//evaluateRand()


   // Evaluate a for-loop expression
   protected Value evaluateFor(Tree tree) throws EvalException
   {

      // Your implementation goes here.

      return new Value( false );
   }//evaluateFor()


   // Evaluate an increment/decrement expression
   protected Value evaluateIncDec(Tree tree) throws EvalException
   {
      Value result = null;

      // Your implementation goes here.

      return result;
   }//evaluateIncDec()


   // Evaluate a set expression
   protected Value evaluateSet(Tree tree) throws EvalException
   {
      Value result = null;

      // Your implementation goes here.

      return result;
   }//evaluateSet()

}//Language_7a